@@ -39,17 +39,11 @@ access results
 # More questions?
 
 If you have any questions, feel free to post a bug report/question on the
-Metaflow Github page.
+Metaflow GitHub page.
 """
 
-import importlib
+import os
 import sys
-import types
-
-from os import path
-
-CURRENT_DIRECTORY = path.dirname(path.abspath(__file__))
-INFO_FILE = path.join(path.dirname(CURRENT_DIRECTORY), "INFO")
 
 from metaflow.extension_support import (
     alias_submodules,
@@ -61,7 +55,6 @@ from metaflow.extension_support import (
     _ext_debug,
 )
 
-
 # We load the module overrides *first* explicitly. Non overrides can be loaded
 # in toplevel as well but these can be loaded first if needed. Note that those
 # modules should be careful not to include anything in Metaflow at their top-level
@@ -79,9 +72,14 @@ try:
             )
         tl_module = m.module.__dict__.get("toplevel", None)
         if tl_module is not None:
-            _tl_modules.append(".".join([EXT_PKG, m.tl_package, "toplevel", tl_module]))
+            _tl_modules.append(
+                (
+                    m.package_name,
+                    ".".join([EXT_PKG, m.tl_package, "toplevel", tl_module]),
+                )
+            )
     _ext_debug("Got overrides to load: %s" % _override_modules)
-    _ext_debug("Got top-level imports: %s" % _tl_modules)
+    _ext_debug("Got top-level imports: %s" % str(_tl_modules))
 except Exception as e:
     _ext_debug("Error in importing toplevel/overrides: %s" % e)
 
@@ -98,7 +96,7 @@ from .multicore_utils import parallel_imap_unordered, parallel_map
 from .metaflow_profile import profile
 
 # current runtime singleton
-from .current import current
+from .metaflow_current import current
 
 # Flow spec
 from .flowspec import FlowSpec
@@ -108,7 +106,11 @@ from .parameters import Parameter, JSONTypeClass
 JSONType = JSONTypeClass()
 
 # data layer
-from .datatools import S3
+# For historical reasons, we make metaflow.plugins.datatools accessible as
+# metaflow.datatools. S3 is also a tool that has historically been available at the
+# top-level so keep as is.
+lazy_load_aliases({"metaflow.datatools": "metaflow.plugins.datatools"})
+from .plugins.datatools import S3
 
 # includefile
 from .includefile import IncludeFile
@@ -139,9 +141,19 @@ from .client import (
     DataArtifact,
 )
 
-__version_addl__ = []
+# Import data class within tuple_util but not introduce new symbols.
+from . import tuple_util
+
+# Runner API
+if sys.version_info >= (3, 7):
+    from .runner.metaflow_runner import Runner
+    from .runner.nbrun import NBRunner
+    from .runner.deployer import Deployer
+    from .runner.nbdeploy import NBDeployer
+
+__ext_tl_modules__ = []
 _ext_debug("Loading top-level modules")
-for m in _tl_modules:
+for pkg_name, m in _tl_modules:
     extension_module = load_module(m)
     if extension_module:
         tl_package = m.split(".")[1]
@@ -149,15 +161,7 @@ for m in _tl_modules:
         lazy_load_aliases(
             alias_submodules(extension_module, tl_package, None, extra_indent=True)
         )
-        version_info = getattr(extension_module, "__mf_extensions__", "<unk>")
-        if extension_module.__version__:
-            version_info = "%s(%s)" % (version_info, extension_module.__version__)
-        __version_addl__.append(version_info)
-
-if __version_addl__:
-    __version_addl__ = ";".join(__version_addl__)
-else:
-    __version_addl__ = None
+        __ext_tl_modules__.append((pkg_name, extension_module))
 
 # Erase all temporary names to avoid leaking things
 for _n in [
@@ -184,11 +188,6 @@ for _n in [
         pass
 del globals()["_n"]
 
-import pkg_resources
+from .version import metaflow_version as _mf_version
 
-try:
-    __version__ = pkg_resources.get_distribution("metaflow").version
-except:
-    # this happens on remote environments since the job package
-    # does not have a version
-    __version__ = None
+__version__ = _mf_version
