@@ -6,14 +6,12 @@ from metaflow.plugins.cards.card_modules.components import (
     Image,
     Error,
     Markdown,
+    VegaChart,
+    ProgressBar,
 )
 from metaflow.plugins.cards.card_modules.basic import (
     DefaultCard,
     PageComponent,
-    RENDER_TEMPLATE_PATH,
-    TaskToDict,
-    DefaultComponent,
-    TaskInfoComponent,
     ErrorCard,
     BlankCard,
 )
