@@ -16,6 +16,8 @@ from ..util import Path, is_stringish, to_fileobj
 
 from .exceptions import DataException, UnpicklableArtifactException
 
+_included_file_type = "<class 'metaflow.includefile.IncludedFile'>"
+
 
 def only_if_not_done(f):
     @wraps(f)
@@ -150,7 +152,7 @@ class TaskDataStore(object):
                 if self._attempt is None:
                     self._attempt = max_attempt
                 elif max_attempt is None or self._attempt > max_attempt:
-                    # In this case, the attempt does not exist so we can't load
+                    # In this case the attempt does not exist, so we can't load
                     # anything
                     self._objects = {}
                     self._info = {}
@@ -171,6 +173,26 @@ class TaskDataStore(object):
                 if data_obj is not None:
                     self._objects = data_obj.get("objects", {})
                     self._info = data_obj.get("info", {})
+        elif self._mode == "d":
+            self._objects = {}
+            self._info = {}
+
+            if self._attempt is None:
+                for i in range(metaflow_config.MAX_ATTEMPTS):
+                    check_meta = self._metadata_name_for_attempt(
+                        self.METADATA_ATTEMPT_SUFFIX, i
+                    )
+                    if self.has_metadata(check_meta, add_attempt=False):
+                        self._attempt = i
+
+            # Do not allow destructive operations on the datastore if attempt is still in flight
+            # and we explicitly did not allow operating on running tasks.
+            if not allow_not_done and not self.has_metadata(self.METADATA_DONE_SUFFIX):
+                raise DataException(
+                    "No completed attempts of the task was found for task '%s'"
+                    % self._path
+                )
+
         else:
             raise DataException("Unknown datastore mode: '%s'" % self._mode)
 
@@ -299,6 +321,7 @@ class TaskDataStore(object):
                     "type": str(type(obj)),
                     "encoding": encode_type,
                 }
+
                 artifact_names.append(name)
                 yield blob
 
@@ -358,7 +381,7 @@ class TaskDataStore(object):
         # We assume that if we have one "old" style artifact, all of them are
         # like that which is an easy assumption to make since artifacts are all
         # stored by the same implementation of the datastore for a given task.
-        for (key, blob) in self._ca_store.load_blobs(to_load.keys()):
+        for key, blob in self._ca_store.load_blobs(to_load.keys()):
             names = to_load[key]
             for name in names:
                 # We unpickle everytime to have fully distinct objects (the user
@@ -386,7 +409,11 @@ class TaskDataStore(object):
         """
         for name in names:
             info = self._info.get(name)
-            yield name, info.get("size", 0)
+            if info["type"] == _included_file_type:
+                sz = self[name].size
+            else:
+                sz = info.get("size", 0)
+            yield name, sz
 
     @require_mode("r")
     def get_legacy_log_size(self, stream):
@@ -569,7 +596,7 @@ class TaskDataStore(object):
             # Conservatively check if the actual object is None,
             # in case the artifact is stored using a different python version.
             # Note that if an object is None and stored in Py2 and accessed in
-            # Py3, this test will fail and we will fallback to the slow path. This
+            # Py3, this test will fail and we will fall back to the slow path. This
             # is intended (being conservative)
             if obj_type == str(type(None)):
                 return True
@@ -681,7 +708,7 @@ class TaskDataStore(object):
             self._info.update(flow._datastore._info)
 
         # we create a list of valid_artifacts in advance, outside of
-        # artifacts_iter so we can provide a len_hint below
+        # artifacts_iter, so we can provide a len_hint below
         valid_artifacts = []
         for var in dir(flow):
             if var.startswith("__") or var in flow._EPHEMERAL:
@@ -743,6 +770,36 @@ class TaskDataStore(object):
                 to_store_dict[n] = data
         self._save_file(to_store_dict)
 
+    @require_mode("d")
+    def scrub_logs(self, logsources, stream, attempt_override=None):
+        path_logsources = {
+            self._metadata_name_for_attempt(
+                self._get_log_location(s, stream),
+                attempt_override=attempt_override,
+            ): s
+            for s in logsources
+        }
+
+        # Legacy log paths
+        legacy_log = self._metadata_name_for_attempt(
+            "%s.log" % stream, attempt_override
+        )
+        path_logsources[legacy_log] = stream
+
+        existing_paths = [
+            path
+            for path in path_logsources.keys()
+            if self.has_metadata(path, add_attempt=False)
+        ]
+
+        # Replace log contents with [REDACTED source stream]
+        to_store_dict = {
+            path: bytes("[REDACTED %s %s]" % (path_logsources[path], stream), "utf-8")
+            for path in existing_paths
+        }
+
+        self._save_file(to_store_dict, add_attempt=False, allow_overwrite=True)
+
     @require_mode("r")
     def load_log_legacy(self, stream, attempt_override=None):
         """
@@ -783,18 +840,37 @@ class TaskDataStore(object):
                 continue
             if k[0] == "_" and not show_private:
                 continue
-            if max_value_size is not None and self._info[k]["size"] > max_value_size:
-                d[k] = ArtifactTooLarge()
+
+            info = self._info[k]
+            if max_value_size is not None:
+                if info["type"] == _included_file_type:
+                    sz = self[k].size
+                else:
+                    sz = info.get("size", 0)
+
+                if sz == 0 or sz > max_value_size:
+                    d[k] = ArtifactTooLarge()
+                else:
+                    d[k] = self[k]
+                    if info["type"] == _included_file_type:
+                        d[k] = d[k].decode(k)
             else:
                 d[k] = self[k]
+                if info["type"] == _included_file_type:
+                    d[k] = d[k].decode(k)
+
         return d
 
     @require_mode("r")
     def format(self, **kwargs):
         def lines():
             for k, v in self.to_dict(**kwargs).items():
+                if self._info[k]["type"] == _included_file_type:
+                    sz = self[k].size
+                else:
+                    sz = self._info[k]["size"]
                 yield k, "*{key}* [size: {size} type: {type}] = {value}".format(
-                    key=k, value=v, **self._info[k]
+                    key=k, value=v, size=sz, type=self._info[k]["type"]
                 )
 
         return "\n".join(line for k, line in sorted(lines()))
