@@ -2,19 +2,26 @@ import inspect
 import os
 import sys
 import traceback
+import reprlib
 
 from itertools import islice
 from types import FunctionType, MethodType
+from typing import Any, Callable, List, Optional, Tuple
 
-from . import cmd_with_io
-from .parameters import Parameter
+from . import cmd_with_io, parameters
+from .parameters import DelayedEvaluationParameter, Parameter
 from .exception import (
     MetaflowException,
     MissingInMergeArtifactsException,
     UnhandledInMergeArtifactsException,
 )
+
+from .extension_support import extension_info
+
 from .graph import FlowGraph
 from .unbounded_foreach import UnboundedForeachInput
+from .util import to_pod
+from .metaflow_config import INCLUDE_FOREACH_STACK, MAXIMUM_FOREACH_VALUE_CHARS
 
 # For Python 3 compatibility
 try:
@@ -23,6 +30,18 @@ except NameError:
     basestring = str
 
 
+from .datastore.inputs import Inputs
+
+INTERNAL_ARTIFACTS_SET = set(
+    [
+        "_foreach_values",
+        "_unbounded_foreach",
+        "_control_mapper_tasks",
+        "_control_task_is_mapper_zero",
+    ]
+)
+
+
 class InvalidNextException(MetaflowException):
     headline = "Invalid self.next() transition detected"
 
@@ -45,7 +64,18 @@ class ParallelUBF(UnboundedForeachInput):
         return item or 0  # item is None for the control task, but it is also split 0
 
 
-class FlowSpec(object):
+class _FlowSpecMeta(type):
+    def __new__(cls, name, bases, dct):
+        f = super().__new__(cls, name, bases, dct)
+        # This makes sure to give _flow_decorators to each
+        # child class (and not share it with the FlowSpec base
+        # class). This is important to not make a "global"
+        # _flow_decorators
+        f._flow_decorators = {}
+        return f
+
+
+class FlowSpec(metaclass=_FlowSpecMeta):
     """
     Main class from which all Flows should inherit.
 
@@ -75,15 +105,13 @@ class FlowSpec(object):
     # names starting with `_` as those are already excluded from `_get_parameters`.
     _NON_PARAMETERS = {"cmd", "foreach_stack", "index", "input", "script_name", "name"}
 
-    _flow_decorators = {}
-
     def __init__(self, use_cli=True):
         """
         Construct a FlowSpec
 
         Parameters
         ----------
-        use_cli : bool, optional, default: True
+        use_cli : bool, default True
             Set to True if the flow is invoked from __main__ or the command line
         """
 
@@ -97,14 +125,13 @@ class FlowSpec(object):
         self._steps = [getattr(self, node.name) for node in self._graph]
 
         if use_cli:
-            # we import cli here to make sure custom parameters in
-            # args.py get fully evaluated before cli.py is imported.
-            from . import cli
+            with parameters.flow_context(self.__class__) as _:
+                from . import cli
 
-            cli.main(self)
+                cli.main(self)
 
     @property
-    def script_name(self):
+    def script_name(self) -> str:
         """
         [Legacy function - do not use. Use `current` instead]
 
@@ -145,9 +172,8 @@ class FlowSpec(object):
         for var, param in self._get_parameters():
             seen.add(var)
             val = kwargs[param.name.replace("-", "_").lower()]
-            # Support for delayed evaluation of parameters. This is used for
-            # includefile in particular
-            if callable(val):
+            # Support for delayed evaluation of parameters.
+            if isinstance(val, DelayedEvaluationParameter):
                 val = val()
             val = val.split(param.separator) if val and param.separator else val
             setattr(self, var, val)
@@ -179,21 +205,23 @@ class FlowSpec(object):
             "decorators": [
                 {
                     "name": deco.name,
-                    "attributes": deco.attributes,
+                    "attributes": to_pod(deco.attributes),
                     "statically_defined": deco.statically_defined,
                 }
-                for deco in flow_decorators()
+                for deco in flow_decorators(self)
                 if not deco.name.startswith("_")
             ],
+            "extensions": extension_info(),
         }
         self._graph_info = graph_info
 
-    def _get_parameters(self):
-        for var in dir(self):
-            if var[0] == "_" or var in self._NON_PARAMETERS:
+    @classmethod
+    def _get_parameters(cls):
+        for var in dir(cls):
+            if var[0] == "_" or var in cls._NON_PARAMETERS:
                 continue
             try:
-                val = getattr(self, var)
+                val = getattr(cls, var)
             except:
                 continue
             if isinstance(val, Parameter):
@@ -215,7 +243,7 @@ class FlowSpec(object):
         """
         return iter(self._steps)
 
-    def __getattr__(self, name):
+    def __getattr__(self, name: str):
         if self._datastore and name in self._datastore:
             # load the attribute from the datastore...
             x = self._datastore[name]
@@ -232,7 +260,7 @@ class FlowSpec(object):
         return cmd_with_io.cmd(cmdline, input=input, output=output)
 
     @property
-    def index(self):
+    def index(self) -> Optional[int]:
         """
         The index of this foreach branch.
 
@@ -245,14 +273,14 @@ class FlowSpec(object):
 
         Returns
         -------
-        int
+        int, optional
             Index of the task in a foreach step.
         """
         if self._foreach_stack:
             return self._foreach_stack[-1].index
 
     @property
-    def input(self):
+    def input(self) -> Optional[Any]:
         """
         The value of the foreach artifact in this foreach branch.
 
@@ -265,12 +293,12 @@ class FlowSpec(object):
 
         Returns
         -------
-        object
+        object, optional
             Input passed to the foreach task.
         """
         return self._find_input()
 
-    def foreach_stack(self):
+    def foreach_stack(self) -> Optional[List[Tuple[int, int, Any]]]:
         """
         Returns the current stack of foreach indexes and values for the current step.
 
@@ -314,7 +342,7 @@ class FlowSpec(object):
 
         Returns
         -------
-        List[Tuple[int, int, object]]
+        List[Tuple[int, int, Any]]
             An array describing the current stack of foreach steps.
         """
         return [
@@ -354,7 +382,12 @@ class FlowSpec(object):
                     )
             return self._cached_input[stack_index]
 
-    def merge_artifacts(self, inputs, exclude=[], include=[]):
+    def merge_artifacts(
+        self,
+        inputs: Inputs,
+        exclude: Optional[List[str]] = None,
+        include: Optional[List[str]] = None,
+    ) -> None:
         """
         Helper function for merging artifacts in a join step.
 
@@ -387,12 +420,12 @@ class FlowSpec(object):
 
         Parameters
         ----------
-        inputs : List[Steps]
+        inputs : Inputs
             Incoming steps to the join point.
-        exclude : List[str], optional
+        exclude : List[str], optional, default None
             If specified, do not consider merging artifacts with a name in `exclude`.
             Cannot specify if `include` is also specified.
-        include : List[str], optional
+        include : List[str], optional, default None
             If specified, only merge artifacts specified. Cannot specify if `exclude` is
             also specified.
 
@@ -406,6 +439,8 @@ class FlowSpec(object):
             This exception is thrown in case an artifact specified in `include` cannot
             be found.
         """
+        include = include or []
+        exclude = exclude or []
         node = self._graph[self._current_step]
         if node.type != "join":
             msg = (
@@ -431,7 +466,9 @@ class FlowSpec(object):
                 available_vars = (
                     (var, sha)
                     for var, sha in inp._datastore.items()
-                    if (var not in exclude) and (not hasattr(self, var))
+                    if (var not in exclude)
+                    and (not hasattr(self, var))
+                    and (var not in INTERNAL_ARTIFACTS_SET)
                 )
             for var, sha in available_vars:
                 _, previous_sha = to_merge.setdefault(var, (inp, sha))
@@ -444,7 +481,7 @@ class FlowSpec(object):
             if v not in to_merge and not hasattr(self, v):
                 missing.append(v)
         if unresolved:
-            # We have unresolved conflicts so we do not set anything and error out
+            # We have unresolved conflicts, so we do not set anything and error out
             msg = (
                 "Step *{step}* cannot merge the following artifacts due to them "
                 "having conflicting values:\n[{artifacts}].\nTo remedy this issue, "
@@ -489,7 +526,34 @@ class FlowSpec(object):
             )
             raise InvalidNextException(msg)
 
-    def next(self, *dsts, **kwargs):
+    def _get_foreach_item_value(self, item: Any):
+        """
+        Get the unique value for the item in the foreach iterator.  If no suitable value
+        is found, return the value formatted by reprlib, which is at most 30 characters long.
+
+        Parameters
+        ----------
+        item : Any
+            The item to get the value from.
+
+        Returns
+        -------
+        str
+            The value to use for the item.
+        """
+
+        def _is_primitive_type(item):
+            return (
+                isinstance(item, basestring)
+                or isinstance(item, int)
+                or isinstance(item, float)
+                or isinstance(item, bool)
+            )
+
+        value = item if _is_primitive_type(item) else reprlib.Repr().repr(item)
+        return basestring(value)[:MAXIMUM_FOREACH_VALUE_CHARS]
+
+    def next(self, *dsts: Callable[..., None], **kwargs) -> None:
         """
         Indicates the next step to execute after this step has completed.
 
@@ -509,13 +573,13 @@ class FlowSpec(object):
           self.next(self.foreach_step, foreach='foreach_iterator')
           ```
           In this situation, `foreach_step` is a method in the current class decorated with the
-          `@step` docorator and `foreach_iterator` is a variable name in the current class that
+          `@step` decorator and `foreach_iterator` is a variable name in the current class that
           evaluates to an iterator. A task will be launched for each value in the iterator and
           each task will execute the code specified by the step `foreach_step`.
 
         Parameters
         ----------
-        dsts : Method
+        dsts : Callable[..., None]
             One or more methods annotated with `@step`.
 
         Raises
@@ -598,19 +662,26 @@ class FlowSpec(object):
                     )
                 )
                 raise InvalidNextException(msg)
-
+            self._foreach_values = None
             if issubclass(type(foreach_iter), UnboundedForeachInput):
                 self._unbounded_foreach = True
                 self._foreach_num_splits = None
                 self._validate_ubf_step(funcs[0])
             else:
                 try:
-                    self._foreach_num_splits = sum(1 for _ in foreach_iter)
-                except TypeError:
+                    if INCLUDE_FOREACH_STACK:
+                        self._foreach_values = []
+                        for item in foreach_iter:
+                            value = self._get_foreach_item_value(item)
+                            self._foreach_values.append(value)
+                        self._foreach_num_splits = len(self._foreach_values)
+                    else:
+                        self._foreach_num_splits = sum(1 for _ in foreach_iter)
+                except Exception as e:
                     msg = (
                         "Foreach variable *self.{var}* in step *{step}* "
-                        "is not iterable. Check your variable.".format(
-                            step=step, var=foreach
+                        "is not iterable. Please check details: {err}".format(
+                            step=step, var=foreach, err=str(e)
                         )
                     )
                     raise InvalidNextException(msg)
