@@ -11,6 +11,7 @@ from .metaflow_config import DEFAULT_PACKAGE_SUFFIXES
 from .exception import MetaflowException
 from .util import to_unicode
 from . import R
+from .info_file import INFO_FILE
 
 DEFAULT_SUFFIXES_LIST = DEFAULT_PACKAGE_SUFFIXES.split(",")
 METAFLOW_SUFFIXES_LIST = [".py", ".html", ".css", ".js"]
@@ -22,7 +23,7 @@ class NonUniqueFileNameToFilePathMappingException(MetaflowException):
     def __init__(self, filename, file_paths, lineno=None):
         msg = (
             "Filename %s included in the code package includes multiple different paths for the same name : %s.\n"
-            "The `filename` in the `add_to_package` decorator hook requires a unqiue `file_path` to `file_name` mapping"
+            "The `filename` in the `add_to_package` decorator hook requires a unique `file_path` to `file_name` mapping"
             % (filename, ", ".join(file_paths))
         )
         super().__init__(msg=msg, lineno=lineno)
@@ -88,9 +89,10 @@ class MetaflowPackage(object):
             # if path and (path[0] == '.' or './' in path):
             #    continue
             for fname in files:
-                if fname[0] == ".":
-                    continue
-                if any(fname.endswith(suffix) for suffix in suffixes):
+                if (fname[0] == "." and fname in suffixes) or (
+                    fname[0] != "."
+                    and any(fname.endswith(suffix) for suffix in suffixes)
+                ):
                     p = os.path.join(path, fname)
                     yield p, p[prefixlen:]
 
@@ -108,7 +110,10 @@ class MetaflowPackage(object):
 
         # Metaflow extensions; for now, we package *all* extensions but this may change
         # at a later date; it is possible to call `package_mfext_package` instead of
-        # `package_mfext_all`
+        # `package_mfext_all` but in that case, make sure to also add a
+        # metaflow_extensions/__init__.py file to properly "close" the metaflow_extensions
+        # package and prevent other extensions from being loaded that may be
+        # present in the rest of the system
         for path_tuple in package_mfext_all():
             yield path_tuple
 
@@ -147,19 +152,22 @@ class MetaflowPackage(object):
                 yield path_tuple
 
     def _add_info(self, tar):
-        info = tarfile.TarInfo("INFO")
-        env = self.environment.get_environment_info()
+        info = tarfile.TarInfo(os.path.basename(INFO_FILE))
+        env = self.environment.get_environment_info(include_ext_info=True)
         buf = BytesIO()
         buf.write(json.dumps(env).encode("utf-8"))
         buf.seek(0)
         info.size = len(buf.getvalue())
+        # Setting this default to Dec 3, 2019
+        info.mtime = 1575360000
         tar.addfile(info, buf)
 
     def _make(self):
         def no_mtime(tarinfo):
             # a modification time change should not change the hash of
             # the package. Only content modifications will.
-            tarinfo.mtime = 0
+            # Setting this default to Dec 3, 2019
+            tarinfo.mtime = 1575360000
             return tarinfo
 
         buf = BytesIO()
