@@ -1,12 +1,8 @@
-import subprocess
-import os
-import tempfile
-import sys
-import json
-from metaflow.decorators import StepDecorator, flow_decorators
-from metaflow.current import current
+from metaflow.decorators import StepDecorator
+from metaflow.metaflow_current import current
 from metaflow.util import to_unicode
 from .component_serializer import CardComponentCollector, get_card_class
+from .card_creator import CardCreator
 
 
 # from metaflow import get_metadata
@@ -14,6 +10,8 @@ import re
 
 from .exception import CARD_ID_PATTERN, TYPE_CHECK_REGEX
 
+ASYNC_TIMEOUT = 30
+
 
 def warning_message(message, logger=None, ts=False):
     msg = "[@card WARNING] %s" % message
@@ -29,15 +27,29 @@ class CardDecorator(StepDecorator):
 
     Parameters
     ----------
-    type : str
-        Card type (default: 'default').
-    id : str
+    type : str, default 'default'
+        Card type.
+    id : str, optional, default None
         If multiple cards are present, use this id to identify this card.
-    options : Dict
+    options : Dict[str, Any], default {}
         Options passed to the card. The contents depend on the card type.
-    timeout : int
-        Interrupt reporting if it takes more than this many seconds
-        (default: 45).
+    timeout : int, default 45
+        Interrupt reporting if it takes more than this many seconds.
+
+    MF Add To Current
+    -----------------
+    card -> metaflow.plugins.cards.component_serializer.CardComponentCollector
+        The `@card` decorator makes the cards available through the `current.card`
+        object. If multiple `@card` decorators are present, you can add an `ID` to
+        distinguish between them using `@card(id=ID)` as the decorator. You will then
+        be able to access that specific card using `current.card[ID].
+
+        Methods available are `append` and `extend`
+
+        @@ Returns
+        -------
+        CardComponentCollector
+            The or one of the cards attached to this step.
     """
 
     name = "card"
@@ -49,6 +61,7 @@ class CardDecorator(StepDecorator):
         "id": None,
         "save_errors": True,
         "customize": False,
+        "refresh_interval": 5,
     }
     allow_multiple = True
 
@@ -58,6 +71,8 @@ class CardDecorator(StepDecorator):
 
     _called_once = {}
 
+    card_creator = None
+
     def __init__(self, *args, **kwargs):
         super(CardDecorator, self).__init__(*args, **kwargs)
         self._task_datastore = None
@@ -68,6 +83,10 @@ class CardDecorator(StepDecorator):
         self._card_uuid = None
         self._user_set_card_id = None
 
+    @classmethod
+    def _set_card_creator(cls, card_creator):
+        cls.card_creator = card_creator
+
     def _is_event_registered(self, evt_name):
         return evt_name in self._called_once
 
@@ -87,31 +106,17 @@ class CardDecorator(StepDecorator):
     def step_init(
         self, flow, graph, step_name, decorators, environment, flow_datastore, logger
     ):
-
         self._flow_datastore = flow_datastore
         self._environment = environment
         self._logger = logger
         self.card_options = None
 
-        # Populate the defaults which may be missing.
-        missing_keys = set(self.defaults.keys()) - set(self.attributes.keys())
-        for k in missing_keys:
-            self.attributes[k] = self.defaults[k]
-
-        # when instantiation happens from the CLI we sometimes get stringified JSON and sometimes a dict for the
-        # `options` attributes. Hence we need to check for both and serialized.
-        if type(self.attributes["options"]) is str:
-            try:
-                self.card_options = json.loads(self.attributes["options"])
-            except json.decoder.JSONDecodeError:
-                self.card_options = self.defaults["options"]
-        else:
-            self.card_options = self.attributes["options"]
+        self.card_options = self.attributes["options"]
 
         evt_name = "step-init"
         # `'%s-%s'%(evt_name,step_name)` ensures that we capture this once per @card per @step.
         # Since there can be many steps checking if event is registered for `evt_name` will only make it check it once for all steps.
-        # Hence we have `_is_event_registered('%s-%s'%(evt_name,step_name))`
+        # Hence, we have `_is_event_registered('%s-%s'%(evt_name,step_name))`
         evt = "%s-%s" % (evt_name, step_name)
         if not self._is_event_registered(evt):
             # We set the total count of decorators so that we can use it for
@@ -137,11 +142,18 @@ class CardDecorator(StepDecorator):
         ubf_context,
         inputs,
     ):
+        self._task_datastore = task_datastore
+        self._metadata = metadata
+
         card_type = self.attributes["type"]
         card_class = get_card_class(card_type)
+
+        self._is_runtime_card = False
         if card_class is not None:  # Card type was not found
             if card_class.ALLOW_USER_COMPONENTS:
                 self._is_editable = True
+            self._is_runtime_card = card_class.RUNTIME_UPDATABLE
+
         # We have a step counter to ensure that on calling the final card decorator's `task_pre_step`
         # we call a `finalize` function in the `CardComponentCollector`.
         # This can help ensure the behaviour of the `current.card` object is according to specification.
@@ -166,7 +178,11 @@ class CardDecorator(StepDecorator):
         # we need to ensure that `current.card` has `CardComponentCollector` instantiated only once.
         if not self._is_event_registered("pre-step"):
             self._register_event("pre-step")
-            current._update_env({"card": CardComponentCollector(self._logger)})
+            self._set_card_creator(CardCreator(self._create_top_level_args()))
+
+            current._update_env(
+                {"card": CardComponentCollector(self._logger, self.card_creator)}
+            )
 
         # this line happens because of decospecs parsing.
         customize = False
@@ -176,28 +192,35 @@ class CardDecorator(StepDecorator):
         card_metadata = current.card._add_card(
             self.attributes["type"],
             self._user_set_card_id,
-            self._is_editable,
-            customize,
+            self.attributes,
+            self.card_options,
+            editable=self._is_editable,
+            customize=customize,
+            runtime_card=self._is_runtime_card,
+            refresh_interval=self.attributes["refresh_interval"],
         )
         self._card_uuid = card_metadata["uuid"]
 
-        # This means that the we are calling `task_pre_step` on the last card decorator.
+        # This means that we are calling `task_pre_step` on the last card decorator.
         # We can now `finalize` method in the CardComponentCollector object.
-        # This will setup the `current.card` object for usage inside `@step` code.
+        # This will set up the `current.card` object for usage inside `@step` code.
         if self.step_counter == self.total_decos_on_step[step_name]:
             current.card._finalize()
 
-        self._task_datastore = task_datastore
-        self._metadata = metadata
-
     def task_finished(
         self, step_name, flow, graph, is_task_ok, retry_count, max_user_code_retries
     ):
-        if not is_task_ok:
-            return
-        component_strings = current.card._serialize_components(self._card_uuid)
-        runspec = "/".join([current.run_id, current.step_name, current.task_id])
-        self._run_cards_subprocess(runspec, component_strings)
+        create_options = dict(
+            card_uuid=self._card_uuid,
+            user_set_card_id=self._user_set_card_id,
+            runtime_card=self._is_runtime_card,
+            decorator_attributes=self.attributes,
+            card_options=self.card_options,
+            logger=self._logger,
+        )
+        if is_task_ok:
+            self.card_creator.create(mode="render", final=True, **create_options)
+            self.card_creator.create(mode="refresh", final=True, **create_options)
 
     @staticmethod
     def _options(mapping):
@@ -211,79 +234,16 @@ class CardDecorator(StepDecorator):
                         yield to_unicode(value)
 
     def _create_top_level_args(self):
-
         top_level_options = {
             "quiet": True,
             "metadata": self._metadata.TYPE,
             "environment": self._environment.TYPE,
             "datastore": self._flow_datastore.TYPE,
             "datastore-root": self._flow_datastore.datastore_root,
+            "no-pylint": True,
+            "event-logger": "nullSidecarLogger",
+            "monitor": "nullSidecarMonitor",
             # We don't provide --with as all execution is taking place in
-            # the context of the main processs
+            # the context of the main process
         }
         return list(self._options(top_level_options))
-
-    def _run_cards_subprocess(self, runspec, component_strings):
-        temp_file = None
-        if len(component_strings) > 0:
-            temp_file = tempfile.NamedTemporaryFile("w", suffix=".json")
-            json.dump(component_strings, temp_file)
-            temp_file.seek(0)
-        executable = sys.executable
-        cmd = [
-            executable,
-            sys.argv[0],
-        ]
-        cmd += self._create_top_level_args() + [
-            "card",
-            "create",
-            runspec,
-            "--type",
-            self.attributes["type"],
-            # Add the options relating to card arguments.
-            # todo : add scope as a CLI arg for the create method.
-        ]
-        if self.card_options is not None and len(self.card_options) > 0:
-            cmd += ["--options", json.dumps(self.card_options)]
-        # set the id argument.
-
-        if self.attributes["timeout"] is not None:
-            cmd += ["--timeout", str(self.attributes["timeout"])]
-
-        if self._user_set_card_id is not None:
-            cmd += ["--id", str(self._user_set_card_id)]
-
-        # Doing this because decospecs parse information as str, since some non-runtime decorators pass it as bool we parse bool to str
-        if str(self.attributes["save_errors"]) == "True":
-            cmd += ["--render-error-card"]
-
-        if temp_file is not None:
-            cmd += ["--component-file", temp_file.name]
-
-        response, fail = self._run_command(
-            cmd, os.environ, timeout=self.attributes["timeout"]
-        )
-        if fail:
-            resp = "" if response is None else response.decode("utf-8")
-            self._logger(
-                "Card render failed with error : \n\n %s" % resp,
-                timestamp=False,
-                bad=True,
-            )
-
-    def _run_command(self, cmd, env, timeout=None):
-        fail = False
-        timeout_args = {}
-        if timeout is not None:
-            timeout_args = dict(timeout=int(timeout) + 10)
-        try:
-            rep = subprocess.check_output(
-                cmd, env=env, stderr=subprocess.STDOUT, **timeout_args
-            )
-        except subprocess.CalledProcessError as e:
-            rep = e.output
-            fail = True
-        except subprocess.TimeoutExpired as e:
-            rep = e.output
-            fail = True
-        return rep, fail
