@@ -26,9 +26,11 @@ def transform_flow_graph(step_info):
         graph_dict[stepname] = {
             "type": node_to_type(step_info[stepname]["type"]),
             "box_next": step_info[stepname]["type"] not in ("linear", "join"),
-            "box_ends": None
-            if "matching_join" not in step_info[stepname]
-            else step_info[stepname]["matching_join"],
+            "box_ends": (
+                None
+                if "matching_join" not in step_info[stepname]
+                else step_info[stepname]["matching_join"]
+            ),
             "next": step_info[stepname]["next"],
             "doc": step_info[stepname]["doc"],
         }
@@ -42,9 +44,12 @@ def read_file(path):
 
 class DefaultComponent(MetaflowCardComponent):
     """
-    The `DefaultCard` and the `BlankCard` use a JS framework that build the HTML dynamically from JSON. The `DefaultComponent` is the base component that helps build the JSON when `render` is called.
+    The `DefaultCard` and the `BlankCard` use a JS framework that build the HTML dynamically from JSON.
+    The `DefaultComponent` is the base component that helps build the JSON when `render` is called.
 
-    The underlying JS framewok consists of various types of objects. These can be found in: "metaflow/plugins/cards/ui/types.ts". The `type` attribute in a `DefaultComponent` corresponds to the type of component in the Javascript framework.
+    The underlying JS framework consists of various types of objects.
+    These can be found in: "metaflow/plugins/cards/ui/types.ts".
+    The `type` attribute in a `DefaultComponent` corresponds to the type of component in the Javascript framework.
     """
 
     type = None
@@ -143,6 +148,8 @@ class ImageComponent(DefaultComponent):
             label=self._label,
         )
         datadict.update(img_dict)
+        if self.component_id is not None:
+            datadict["id"] = self.component_id
         return datadict
 
 
@@ -191,6 +198,8 @@ class TableComponent(DefaultComponent):
         datadict["columns"] = self._headers
         datadict["data"] = self._data
         datadict["vertical"] = self._vertical
+        if self.component_id is not None:
+            datadict["id"] = self.component_id
         return datadict
 
 
@@ -292,6 +301,8 @@ class ArtifactsComponent(DefaultComponent):
     def render(self):
         datadict = super().render()
         datadict["data"] = self._data
+        if self.component_id is not None:
+            datadict["id"] = self.component_id
         return datadict
 
 
@@ -304,7 +315,16 @@ class MarkdownComponent(DefaultComponent):
 
     def render(self):
         datadict = super().render()
-        datadict["source"] = self._text
+        _text = self._text
+        # When we have a markdown text that doesn't start with a `#`,
+        # we need to add a newline to make sure that the markdown
+        # is rendered correctly. Otherwise `svelte-markdown` will render
+        # the empty `<p>` tags during re-renders on card data updates.
+        if self._text is not None and not self._text.startswith("#"):
+            _text = "\n" + _text
+        datadict["source"] = _text
+        if self.component_id is not None:
+            datadict["id"] = self.component_id
         return datadict
 
 
@@ -316,7 +336,13 @@ class TaskInfoComponent(MetaflowCardComponent):
     """
 
     def __init__(
-        self, task, page_title="Task Info", only_repr=True, graph=None, components=[]
+        self,
+        task,
+        page_title="Task Info",
+        only_repr=True,
+        graph=None,
+        components=[],
+        runtime=False,
     ):
         self._task = task
         self._only_repr = only_repr
@@ -325,6 +351,7 @@ class TaskInfoComponent(MetaflowCardComponent):
         self._page_title = page_title
         self.final_component = None
         self.page_component = None
+        self.runtime = runtime
 
     def render(self):
         """
@@ -337,14 +364,21 @@ class TaskInfoComponent(MetaflowCardComponent):
             self._task, graph=self._graph
         )
         # ignore the name as an artifact
-        del task_data_dict["data"]["name"]
-        mf_version = [
-            t for t in self._task.parent.parent.tags if "metaflow_version" in t
-        ][0].split("metaflow_version:")[1]
+        if "name" in task_data_dict["data"]:
+            del task_data_dict["data"]["name"]
+
+        _metadata = dict(version=1, template="defaultCardTemplate")
+        # try to parse out metaflow version from tags, but let it go if unset
+        # e.g. if a run came from a local, un-versioned metaflow codebase
+        try:
+            _metadata["metaflow_version"] = [
+                t for t in self._task.parent.parent.tags if "metaflow_version" in t
+            ][0].split("metaflow_version:")[1]
+        except Exception:
+            pass
+
         final_component_dict = dict(
-            metadata=dict(
-                metaflow_version=mf_version, version=1, template="defaultCardTemplate"
-            ),
+            metadata=_metadata,
             components=[],
         )
 
@@ -361,11 +395,12 @@ class TaskInfoComponent(MetaflowCardComponent):
             "Task Created On": task_data_dict["created_at"],
             "Task Finished On": task_data_dict["finished_at"],
             # Remove Microseconds from timedelta
-            "Task Duration": str(self._task.finished_at - self._task.created_at).split(
-                "."
-            )[0],
             "Tags": ", ".join(tags),
         }
+        if not self.runtime:
+            task_metadata_dict["Task Duration"] = str(
+                self._task.finished_at - self._task.created_at
+            ).split(".")[0]
         if len(user_info) > 0:
             task_metadata_dict["User"] = user_info[0].split("user:")[1]
 
@@ -425,8 +460,12 @@ class TaskInfoComponent(MetaflowCardComponent):
             p.id for p in self._task.parent.parent["_parameters"].task if p.id != "name"
         ]
         if len(param_ids) > 0:
+            # Extract parameter from the Parameter Task. That is less brittle.
+            parameter_data = TaskToDict(
+                only_repr=self._only_repr, runtime=self.runtime
+            )(self._task.parent.parent["_parameters"].task, graph=self._graph)
             param_component = ArtifactsComponent(
-                data=[task_data_dict["data"][pid] for pid in param_ids]
+                data=[parameter_data["data"][pid] for pid in param_ids]
             )
         else:
             param_component = TitleComponent(text="No Parameters")
@@ -443,12 +482,12 @@ class TaskInfoComponent(MetaflowCardComponent):
             if k not in param_ids
         ]
         if len(artifactlist) > 0:
-            artrifact_component = ArtifactsComponent(data=artifactlist).render()
+            artifact_component = ArtifactsComponent(data=artifactlist).render()
         else:
-            artrifact_component = TitleComponent(text="No Artifacts")
+            artifact_component = TitleComponent(text="No Artifacts")
 
         artifact_section = SectionComponent(
-            title="Artifacts", contents=[artrifact_component]
+            title="Artifacts", contents=[artifact_component]
         ).render()
         dag_component = SectionComponent(
             title="DAG", contents=[DagComponent(data=task_data_dict["graph"]).render()]
@@ -505,11 +544,24 @@ class ErrorCard(MetaflowCard):
 
     type = "error"
 
+    RELOAD_POLICY = MetaflowCard.RELOAD_POLICY_ONCHANGE
+
     def __init__(self, options={}, components=[], graph=None):
         self._only_repr = True
         self._graph = None if graph is None else transform_flow_graph(graph)
         self._components = components
 
+    def reload_content_token(self, task, data):
+        """
+        The reload token will change when the component array has changed in the Metaflow card.
+        The change in the component array is signified by the change in the component_update_ts.
+        """
+        if task.finished:
+            return "final"
+        # `component_update_ts` will never be None. It is set to a default value when the `ComponentStore` is instantiated
+        # And it is updated when components added / removed / changed from the `ComponentStore`.
+        return "runtime-%s" % (str(data["component_update_ts"]))
+
     def render(self, task, stack_trace=None):
         RENDER_TEMPLATE = read_file(RENDER_TEMPLATE_PATH)
         JS_DATA = read_file(JS_PATH)
@@ -571,6 +623,10 @@ class DefaultCard(MetaflowCard):
 
     ALLOW_USER_COMPONENTS = True
 
+    RUNTIME_UPDATABLE = True
+
+    RELOAD_POLICY = MetaflowCard.RELOAD_POLICY_ONCHANGE
+
     type = "default"
 
     def __init__(self, options=dict(only_repr=True), components=[], graph=None):
@@ -580,7 +636,7 @@ class DefaultCard(MetaflowCard):
             self._only_repr = options["only_repr"]
         self._components = components
 
-    def render(self, task):
+    def render(self, task, runtime=False):
         RENDER_TEMPLATE = read_file(RENDER_TEMPLATE_PATH)
         JS_DATA = read_file(JS_PATH)
         CSS_DATA = read_file(CSS_PATH)
@@ -589,6 +645,7 @@ class DefaultCard(MetaflowCard):
             only_repr=self._only_repr,
             graph=self._graph,
             components=self._components,
+            runtime=runtime,
         ).render()
         pt = self._get_mustache()
         data_dict = dict(
@@ -599,14 +656,36 @@ class DefaultCard(MetaflowCard):
             title=task.pathspec,
             css=CSS_DATA,
             card_data_id=uuid.uuid4(),
+            RENDER_COMPLETE=not runtime,
         )
         return pt.render(RENDER_TEMPLATE, data_dict)
 
+    def render_runtime(self, task, data):
+        return self.render(task, runtime=True)
+
+    def refresh(self, task, data):
+        return data["components"]
+
+    def reload_content_token(self, task, data):
+        """
+        The reload token will change when the component array has changed in the Metaflow card.
+        The change in the component array is signified by the change in the component_update_ts.
+        """
+        if task.finished:
+            return "final"
+        # `component_update_ts` will never be None. It is set to a default value when the `ComponentStore` is instantiated
+        # And it is updated when components added / removed / changed from the `ComponentStore`.
+        return "runtime-%s" % (str(data["component_update_ts"]))
+
 
 class BlankCard(MetaflowCard):
 
     ALLOW_USER_COMPONENTS = True
 
+    RUNTIME_UPDATABLE = True
+
+    RELOAD_POLICY = MetaflowCard.RELOAD_POLICY_ONCHANGE
+
     type = "blank"
 
     def __init__(self, options=dict(title=""), components=[], graph=None):
@@ -616,7 +695,7 @@ class BlankCard(MetaflowCard):
             self._title = options["title"]
         self._components = components
 
-    def render(self, task, components=[]):
+    def render(self, task, components=[], runtime=False):
         RENDER_TEMPLATE = read_file(RENDER_TEMPLATE_PATH)
         JS_DATA = read_file(JS_PATH)
         CSS_DATA = read_file(CSS_PATH)
@@ -641,9 +720,27 @@ class BlankCard(MetaflowCard):
             title=task.pathspec,
             css=CSS_DATA,
             card_data_id=uuid.uuid4(),
+            RENDER_COMPLETE=not runtime,
         )
         return pt.render(RENDER_TEMPLATE, data_dict)
 
+    def render_runtime(self, task, data):
+        return self.render(task, runtime=True)
+
+    def refresh(self, task, data):
+        return data["components"]
+
+    def reload_content_token(self, task, data):
+        """
+        The reload token will change when the component array has changed in the Metaflow card.
+        The change in the component array is signified by the change in the component_update_ts.
+        """
+        if task.finished:
+            return "final"
+        # `component_update_ts` will never be None. It is set to a default value when the `ComponentStore` is instantiated
+        # And it is updated when components added / removed / changed from the `ComponentStore`.
+        return "runtime-%s" % (str(data["component_update_ts"]))
+
 
 class TaskSpecCard(MetaflowCard):
     type = "taskspec_card"
