@@ -21,30 +21,21 @@ class FailureHandledByCatch(MetaflowException):
 
 class CatchDecorator(StepDecorator):
     """
-    Step decorator to specify error handling for your step.
+    Specifies that the step will success under all circumstances.
 
-    This decorator indicates that exceptions in the step should be caught and not fail the entire
-    flow.
-
-    This can be used in conjunction with the @retry decorator. In that case, catch will only
-    activate if all retries fail and will catch the last exception thrown by the last retry.
-
-    To use, annotate your step as follows:
-    ```
-    @catch(var='foo')
-    @step
-    def myStep(self):
-        ...
-    ```
+    The decorator will create an optional artifact, specified by `var`, which
+    contains the exception raised. You can use it to detect the presence
+    of errors, indicating that all happy-path artifacts produced by the step
+    are missing.
 
     Parameters
     ----------
-    var : string
-        Name of the artifact in which to store the caught exception. If not specified,
-        the exception is not stored
-    print_exception : bool
-        Determines whether or not the exception is printed to stdout when caught. Defaults
-        to True
+    var : str, optional, default None
+        Name of the artifact in which to store the caught exception.
+        If not specified, the exception is not stored.
+    print_exception : bool, default True
+        Determines whether or not the exception is printed to
+        stdout when caught.
     """
 
     name = "catch"
@@ -74,7 +65,6 @@ class CatchDecorator(StepDecorator):
     def task_exception(
         self, exception, step, flow, graph, retry_count, max_user_code_retries
     ):
-
         # Only "catch" exceptions after all retries are exhausted
         if retry_count < max_user_code_retries:
             return False
@@ -107,9 +97,8 @@ class CatchDecorator(StepDecorator):
         return 0, NUM_FALLBACK_RETRIES
 
     def task_decorate(
-        self, step_func, func, graph, retry_count, max_user_code_retries, ubf_context
+        self, step_func, flow, graph, retry_count, max_user_code_retries, ubf_context
     ):
-
         # if the user code has failed max_user_code_retries times, @catch
         # runs a piece of fallback code instead. This way we can continue
         # running the flow downstream, as we have a proper entry for this task.
