@@ -51,7 +51,7 @@ from .client_modules import create_modules
 # In the case of a bootstrap code (for example on Batch), the subprocess mechanism is
 # not used to determine the outside environment but the `generate_trampolines` function
 # in this file is called directly by the remote bootstrap code which operates *outside*
-# of the created conda environment so it has the same effect
+# of the created conda environment, so it has the same effect
 ENV_ESCAPE_PY = os.environ.get("METAFLOW_ENV_ESCAPE_PY", sys.executable)
 
 _cur_sys_paths = sys.path
@@ -84,10 +84,12 @@ def generate_trampolines(out_dir):
         return
 
     paths = [os.path.join(os.path.dirname(os.path.abspath(__file__)), "configurations")]
-    for m in get_modules("plugins.env_escape"):
-        paths.extend(
-            [os.path.join(x, "configurations") for x in list(m.module.__path__)]
-        )
+    paths.extend(
+        [
+            os.path.join(os.path.dirname(m.module.__file__), "configurations")
+            for m in get_modules("plugins.env_escape")
+        ]
+    )
 
     for rootpath in paths:
         for path in os.listdir(rootpath):
@@ -108,6 +110,7 @@ import importlib
 import os
 import sys
 from metaflow.plugins.env_escape.client_modules import ModuleImporter
+from metaflow.metaflow_config import ESCAPE_HATCH_WARNING
 
 # This is a trampoline file to ensure that the ModuleImporter to handle the emulated
 # modules gets properly loaded. If multiple modules are emulated by a single configuration
@@ -144,7 +147,7 @@ def load():
             # print("Env escape using executable {python_executable}")
         else:
             # Inverse logic as above here.
-            if sys.executable != "{python_executable}":
+            if sys.executable != "{python_executable}" and ESCAPE_HATCH_WARNING:
                 # We use the package locally and warn user.
                 print("Not using environment escape for '%s' as module present" % prefix)
             # In both cases, we don't load our loader since
