@@ -13,10 +13,22 @@ def get_methods(class_object):
     for base_class in mros:
         all_attributes.update(base_class.__dict__)
     for name, attribute in all_attributes.items():
-        if hasattr(attribute, "__call__"):
-            all_methods[name] = inspect.getdoc(attribute)
-        elif isinstance(attribute, staticmethod):
+        if isinstance(attribute, staticmethod):
             all_methods["___s___%s" % name] = inspect.getdoc(attribute)
         elif isinstance(attribute, classmethod):
             all_methods["___c___%s" % name] = inspect.getdoc(attribute)
+        elif hasattr(attribute, "__call__"):
+            all_methods[name] = inspect.getdoc(attribute)
     return all_methods
+
+
+def get_canonical_name(name, aliases):
+    # We look at the aliases looking for the most specific match first
+    base_name = aliases.get(name)
+    if base_name is not None:
+        return base_name
+    for idx in reversed([pos for pos, char in enumerate(name) if char == "."]):
+        base_name = aliases.get(name[:idx])
+        if base_name is not None:
+            return ".".join([base_name, name[idx + 1 :]])
+    return name
