@@ -4,22 +4,22 @@ import random
 import requests
 import time
 
-from distutils.version import LooseVersion
-
 from metaflow.exception import (
     MetaflowException,
     MetaflowTaggingError,
     MetaflowInternalError,
 )
 from metaflow.metaflow_config import (
-    METADATA_SERVICE_NUM_RETRIES,
-    METADATA_SERVICE_HEADERS,
-    METADATA_SERVICE_URL,
+    SERVICE_RETRY_COUNT,
+    SERVICE_HEADERS,
+    SERVICE_URL,
 )
 from metaflow.metadata import MetadataProvider
 from metaflow.metadata.heartbeat import HB_URL_KEY
 from metaflow.sidecar import Message, MessageTypes, Sidecar
 
+from metaflow.util import version_parse
+
 
 # Define message enums
 class HeartbeatTypes(object):
@@ -47,11 +47,11 @@ class ServiceMetadataProvider(MetadataProvider):
             environment, flow, event_logger, monitor
         )
         self.url_task_template = os.path.join(
-            METADATA_SERVICE_URL,
+            SERVICE_URL,
             "flows/{flow_id}/runs/{run_number}/steps/{step_name}/tasks/{task_id}/heartbeat",
         )
         self.url_run_template = os.path.join(
-            METADATA_SERVICE_URL, "flows/{flow_id}/runs/{run_number}/heartbeat"
+            SERVICE_URL, "flows/{flow_id}/runs/{run_number}/heartbeat"
         )
         self.sidecar = None
 
@@ -59,9 +59,7 @@ class ServiceMetadataProvider(MetadataProvider):
     def compute_info(cls, val):
         v = val.rstrip("/")
         try:
-            resp = requests.get(
-                os.path.join(v, "ping"), headers=METADATA_SERVICE_HEADERS
-            )
+            resp = requests.get(os.path.join(v, "ping"), headers=SERVICE_HEADERS.copy())
             resp.raise_for_status()
         except:  # noqa E722
             raise ValueError("Metaflow service [%s] unreachable." % v)
@@ -69,7 +67,7 @@ class ServiceMetadataProvider(MetadataProvider):
 
     @classmethod
     def default_info(cls):
-        return METADATA_SERVICE_URL
+        return SERVICE_URL
 
     def version(self):
         return self._version(self._monitor)
@@ -139,9 +137,10 @@ class ServiceMetadataProvider(MetadataProvider):
             payload[HB_URL_KEY] = self.url_run_template.format(**data)
         else:
             raise Exception("invalid heartbeat type")
-        payload["service_version"] = self.version()
+        service_version = self.version()
+        payload["service_version"] = service_version
         # start sidecar
-        if self.version() is None or LooseVersion(self.version()) < LooseVersion(
+        if service_version is None or version_parse(service_version) < version_parse(
             "2.0.4"
         ):
             # if old version of the service is running
@@ -191,9 +190,9 @@ class ServiceMetadataProvider(MetadataProvider):
         min_service_version_with_tag_mutation = "2.3.0"
         if cls._supports_tag_mutation is None:
             version = cls._version(None)
-            cls._supports_tag_mutation = version is not None and LooseVersion(
+            cls._supports_tag_mutation = version is not None and version_parse(
                 version
-            ) >= LooseVersion(min_service_version_with_tag_mutation)
+            ) >= version_parse(min_service_version_with_tag_mutation)
         if not cls._supports_tag_mutation:
             raise ServiceException(
                 "Adding or removing tags on a run requires the Metaflow service to be "
@@ -240,9 +239,9 @@ class ServiceMetadataProvider(MetadataProvider):
         if attempt is not None:
             if cls._supports_attempt_gets is None:
                 version = cls._version(None)
-                cls._supports_attempt_gets = version is not None and LooseVersion(
+                cls._supports_attempt_gets = version is not None and version_parse(
                     version
-                ) >= LooseVersion("2.0.6")
+                ) >= version_parse("2.0.6")
             if not cls._supports_attempt_gets:
                 raise ServiceException(
                     "Getting specific attempts of Tasks or Artifacts requires "
@@ -348,7 +347,6 @@ class ServiceMetadataProvider(MetadataProvider):
         tags=None,
         sys_tags=None,
     ):
-
         if tags is None:
             tags = set()
         if sys_tags is None:
@@ -409,33 +407,33 @@ class ServiceMetadataProvider(MetadataProvider):
                 % (supported_methods, method)
             )
         url = os.path.join(cls.INFO, path.lstrip("/"))
-        for i in range(METADATA_SERVICE_NUM_RETRIES):
+        for i in range(SERVICE_RETRY_COUNT):
             try:
                 if method == "GET":
                     if monitor:
                         with monitor.measure("metaflow.service_metadata.get"):
-                            resp = requests.get(url, headers=METADATA_SERVICE_HEADERS)
+                            resp = requests.get(url, headers=SERVICE_HEADERS.copy())
                     else:
-                        resp = requests.get(url, headers=METADATA_SERVICE_HEADERS)
+                        resp = requests.get(url, headers=SERVICE_HEADERS.copy())
                 elif method == "POST":
                     if monitor:
                         with monitor.measure("metaflow.service_metadata.post"):
                             resp = requests.post(
-                                url, headers=METADATA_SERVICE_HEADERS, json=data
+                                url, headers=SERVICE_HEADERS.copy(), json=data
                             )
                     else:
                         resp = requests.post(
-                            url, headers=METADATA_SERVICE_HEADERS, json=data
+                            url, headers=SERVICE_HEADERS.copy(), json=data
                         )
                 elif method == "PATCH":
                     if monitor:
                         with monitor.measure("metaflow.service_metadata.patch"):
                             resp = requests.patch(
-                                url, headers=METADATA_SERVICE_HEADERS, json=data
+                                url, headers=SERVICE_HEADERS.copy(), json=data
                             )
                     else:
                         resp = requests.patch(
-                            url, headers=METADATA_SERVICE_HEADERS, json=data
+                            url, headers=SERVICE_HEADERS.copy(), json=data
                         )
                 else:
                     raise MetaflowInternalError("Unexpected HTTP method %s" % (method,))
@@ -444,10 +442,10 @@ class ServiceMetadataProvider(MetadataProvider):
             except:  # noqa E722
                 if monitor:
                     with monitor.count("metaflow.service_metadata.failed_request"):
-                        if i == METADATA_SERVICE_NUM_RETRIES - 1:
+                        if i == SERVICE_RETRY_COUNT - 1:
                             raise
                 else:
-                    if i == METADATA_SERVICE_NUM_RETRIES - 1:
+                    if i == SERVICE_RETRY_COUNT - 1:
                         raise
                 resp = None
             else:
@@ -476,7 +474,7 @@ class ServiceMetadataProvider(MetadataProvider):
                         resp.status_code,
                         resp.text,
                     )
-            time.sleep(2 ** i)
+            time.sleep(2**i)
 
         if resp:
             raise ServiceException(
@@ -497,20 +495,20 @@ class ServiceMetadataProvider(MetadataProvider):
             )
         path = "ping"
         url = os.path.join(cls.INFO, path)
-        for i in range(METADATA_SERVICE_NUM_RETRIES):
+        for i in range(SERVICE_RETRY_COUNT):
             try:
                 if monitor:
                     with monitor.measure("metaflow.service_metadata.get"):
-                        resp = requests.get(url, headers=METADATA_SERVICE_HEADERS)
+                        resp = requests.get(url, headers=SERVICE_HEADERS.copy())
                 else:
-                    resp = requests.get(url, headers=METADATA_SERVICE_HEADERS)
+                    resp = requests.get(url, headers=SERVICE_HEADERS.copy())
             except:
                 if monitor:
                     with monitor.count("metaflow.service_metadata.failed_request"):
-                        if i == METADATA_SERVICE_NUM_RETRIES - 1:
+                        if i == SERVICE_RETRY_COUNT - 1:
                             raise
                 else:
-                    if i == METADATA_SERVICE_NUM_RETRIES - 1:
+                    if i == SERVICE_RETRY_COUNT - 1:
                         raise
                 resp = None
             else:
@@ -523,7 +521,7 @@ class ServiceMetadataProvider(MetadataProvider):
                         resp.status_code,
                         resp.text,
                     )
-            time.sleep(2 ** i)
+            time.sleep(2**i)
         if resp:
             raise ServiceException(
                 "Metadata request (%s) failed (code %s): %s"
