@@ -6,7 +6,7 @@ class ResourcesDecorator(StepDecorator):
     Specifies the resources needed when executing this step.
 
     Use `@resources` to specify the resource requirements
-    independently from the specific compute layer (`@batch`, `@kubernetes`).
+    independently of the specific compute layer (`@batch`, `@kubernetes`).
 
     You can choose the compute layer on the command line by executing e.g.
     ```
@@ -21,16 +21,24 @@ class ResourcesDecorator(StepDecorator):
 
     Parameters
     ----------
-    cpu : int
-        Number of CPUs required for this step. Defaults to 1.
-    gpu : int
-        Number of GPUs required for this step. Defaults to 0.
-    memory : int
-        Memory size (in MB) required for this step. Defaults to 4096.
-    shared_memory : int
+    cpu : int, default 1
+        Number of CPUs required for this step.
+    gpu : int, default 0
+        Number of GPUs required for this step.
+    disk : int, optional, default None
+        Disk size (in MB) required for this step. Only applies on Kubernetes.
+    memory : int, default 4096
+        Memory size (in MB) required for this step.
+    shared_memory : int, optional, default None
         The value for the size (in MiB) of the /dev/shm volume for this step.
         This parameter maps to the `--shm-size` option in Docker.
     """
 
     name = "resources"
-    defaults = {"cpu": "1", "gpu": "0", "memory": "4096", "shared_memory": None}
+    defaults = {
+        "cpu": "1",
+        "gpu": "0",
+        "disk": None,
+        "memory": "4096",
+        "shared_memory": None,
+    }
