@@ -4,7 +4,7 @@ from itertools import chain
 
 from metaflow import namespace
 from metaflow.client import Flow, Run
-from metaflow.current import current
+from metaflow.metaflow_current import current
 from metaflow.util import resolve_identity
 from metaflow.exception import CommandException, MetaflowNotFound, MetaflowInternalError
 from metaflow.exception import MetaflowNamespaceMismatch
@@ -67,7 +67,7 @@ def _print_tags_for_group(
         # We want 120 >= column_count*max_length + (column_count - 1)*4
         column_count = 124 // (max_length + 4)
         if column_count == 0:
-            # Make sure we have at least 1 column even for very very long tags
+            # Make sure we have at least 1 column even for very, very long tags
             column_count = 1
             words_per_column = num_tags
         else:
@@ -120,11 +120,11 @@ def _print_tags_for_group_by_tag(obj, group, runs, is_system):
 
         num_runs = len(all_runs)
 
-        # We consider 4 spaces in between column and we consider 120 characters total
-        # width and we want 120 >= column_count*max_length + (column_count - 1)*4
+        # We consider 4 spaces in between columns, 120 characters total width, and
+        # 120 >= column_count*max_length + (column_count - 1)*4
         column_count = 124 // (max_length + 4)
         if column_count == 0:
-            # Make sure we have at least 1 column even for very very long tags
+            # Make sure we have at least 1 column even for very, very long tags
             column_count = 1
             words_per_column = num_runs
         else:
@@ -507,9 +507,9 @@ def tag_list(
 
     if not group_by_run and not group_by_tag:
         # We list all the runs that match to print them out if needed.
-        system_tags_by_some_grouping[
-            ",".join(pathspecs)
-        ] = system_tags_by_some_grouping.get("_", set())
+        system_tags_by_some_grouping[",".join(pathspecs)] = (
+            system_tags_by_some_grouping.get("_", set())
+        )
         all_tags_by_some_grouping[",".join(pathspecs)] = all_tags_by_some_grouping.get(
             "_", set()
         )
