@@ -8,8 +8,14 @@ import sys
 from metaflow.cli_args import cli_args
 from metaflow.decorators import StepDecorator
 from metaflow.exception import MetaflowException
-from metaflow.unbounded_foreach import UnboundedForeachInput, UBF_CONTROL, UBF_TASK
+from metaflow.unbounded_foreach import (
+    UnboundedForeachInput,
+    UBF_CONTROL,
+    UBF_TASK,
+    CONTROL_TASK_TAG,
+)
 from metaflow.util import to_unicode
+from metaflow.metadata import MetaDatum
 
 
 class InternalTestUnboundedForeachInput(UnboundedForeachInput):
@@ -60,13 +66,42 @@ class InternalTestUnboundedForeachDecorator(StepDecorator):
     ):
         self.environment = environment
 
+    def task_pre_step(
+        self,
+        step_name,
+        task_datastore,
+        metadata,
+        run_id,
+        task_id,
+        flow,
+        graph,
+        retry_count,
+        max_user_code_retries,
+        ubf_context,
+        inputs,
+    ):
+        if ubf_context == UBF_CONTROL:
+            metadata.register_metadata(
+                run_id,
+                step_name,
+                task_id,
+                [
+                    MetaDatum(
+                        field="internal_task_type",
+                        value=CONTROL_TASK_TAG,
+                        type="internal_task_type",
+                        tags=["attempt_id:{0}".format(0)],
+                    )
+                ],
+            )
+            self.input_paths = [obj.pathspec for obj in inputs]
+
     def control_task_step_func(self, flow, graph, retry_count):
         from metaflow import current
 
         run_id = current.run_id
         step_name = current.step_name
         control_task_id = current.task_id
-        (_, split_step_name, split_task_id) = control_task_id.split("-")[1:]
         # If we are running inside Conda, we use the base executable FIRST;
         # the conda environment will then be used when runtime_step_cli is
         # called. This is so that it can properly set up all the metaflow
@@ -94,10 +129,10 @@ class InternalTestUnboundedForeachDecorator(StepDecorator):
         mapper_tasks = []
 
         for i in range(foreach_num_splits):
-            task_id = "%s-%d" % (control_task_id.replace("control-", "test-ubf-"), i)
+            task_id = "%s-%d" % (control_task_id, i)
             pathspec = "%s/%s/%s" % (run_id, step_name, task_id)
             mapper_tasks.append(to_unicode(pathspec))
-            input_paths = "%s/%s/%s" % (run_id, split_step_name, split_task_id)
+            input_paths = ",".join(self.input_paths)
 
             # Override specific `step` kwargs.
             kwargs = cli_args.step_kwargs
@@ -111,11 +146,11 @@ class InternalTestUnboundedForeachDecorator(StepDecorator):
             cmd = cli_args.step_command(
                 executable, script, step_name, step_kwargs=kwargs
             )
-            step_cli = u" ".join(cmd)
+            step_cli = " ".join(cmd)
             # Print cmdline for execution. Doesn't work without the temporary
             # unicode object while using `print`.
             print(
-                u"[${cwd}] Starting split#{split} with cmd:{cmd}".format(
+                "[${cwd}] Starting split#{split} with cmd:{cmd}".format(
                     cwd=os.getcwd(), split=i, cmd=step_cli
                 )
             )
